/*
 * http_server_file.c
 *
 *  Created on: 28.8.2012
 *      Author: velek
 */


#include "platform.h"
#include <microhttpd.h>
#include <unistd.h>
#include <stdbool.h>
#include <string.h>
#include <stdint.h>
#include <stdio.h>

#include <http_server_file.h>

#define MHD_CREATE_RESPONSE_FROM_CALLBACK_BLOCK_SIZE	(512)

/* Tag handler function if not defined, it will copy the tag into buffer*/
static void file_callback_cleanup(void *cls)
{
	//cast the to file descriptor
	FILE * file = (FILE *) cls;

	if (NULL != file)
	{
		// Close the file.
		fclose(file);
	}
	return;
}

/* Only buffer the data */
struct MHD_Response * http_server_send_data(const uint8_t *file_content, size_t file_size, bool rom)
{
	struct MHD_Response *response;
	response = MHD_create_response_from_buffer (file_size, (void *) file_content, (true == rom) ? MHD_RESPMEM_PERSISTENT : MHD_RESPMEM_MUST_FREE);
	return response;
}



/* Read data from file descriptor*/
static ssize_t file_content_generator(void *cls, uint64_t pos, char *buf,
		size_t buffer_len)
{
	(void) pos;
	ssize_t size_to_return = MHD_CONTENT_READER_END_WITH_ERROR;

	// Cast the closure to FILE pointer.
	FILE *file = (FILE *) cls;

	if (NULL != file)
	{
		// Read data to buffer.
		size_to_return = (ssize_t)fread(buf, 1, buffer_len, file);

		// sanity check.
		if (size_to_return <= 0)
		{
			// If end of file, return appropriate value
			if (0 != feof(file))
			{
				// End of file
				size_to_return = MHD_CONTENT_READER_END_OF_STREAM;
			}
			else
			{
				// Some error
				size_to_return = MHD_CONTENT_READER_END_WITH_ERROR;
			}
#if 0 // Now handled in clean up function
			// Close the file.
					fclose(file);
#endif
			// Return some kind of CONTENT_READER_END
			return size_to_return;
		}
	}

	// Return size of read bytes. Sanity check, see above.
	return size_to_return;


}

#if 0
WEB_FILE_DESCRIPTOR * http_file_callback_setup(const char * file_name)
{
	FILE * file = fopen (file_name, "rb");
	return (WEB_FILE_DESCRIPTOR *)file;
}
#endif

struct MHD_Response * http_server_file(const char * file_name)
{

	struct MHD_Response *response = NULL;

	// Open the given file name
	FILE * file = fopen (file_name, "rb");

	if (NULL != file)
	{
		// Create response.
		response = MHD_create_response_from_callback(MHD_SIZE_UNKNOWN,
				MHD_CREATE_RESPONSE_FROM_CALLBACK_BLOCK_SIZE,
				file_content_generator, (void *) file, file_callback_cleanup);

		// If response is null, close file
		if (NULL == response)
		{
			fclose(file);
		}
	}

	return response;
}


